#pragma once
/************************************************************************
    MeOS - Orienteering Software
    Copyright (C) 2009-2011 Melin Software HB
    
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Melin Software HB - software@melin.nu - www.melin.nu
    Stigbergsvgen 7, SE-75242 UPPSALA, Sweden
    
************************************************************************/

#include "tabbase.h"
#include "oEventDraw.h"

class TabClass :
	public TabBase
{
  struct PursuitSettings {
    bool use;
    int firstTime;
    int maxTime;

    PursuitSettings(oClass &c) {
      firstTime = 3600;
      use = c.interpretClassType() != ctOpen;
      maxTime = 3600;
    }
  };

  map<int, PursuitSettings> pSettings;

  enum DrawMethod {
    NOMethod = -1,

    DMRandom = 1,
    DMSOFT = 2,
    DMClumped = 3,

    DMPursuit = 11,
    DMReversePursuit = 12,
    DMSeeded = 13
  };

  bool EditChanged;
  int ClassId;
  int currentStage;
  string storedNStage;
  string storedStart;
  oEvent::PredefinedTypes storedPredefined;

  bool checkClassSelected(const gdioutput &gdi) const;
  void save(gdioutput &gdi, bool skipReload);
  void legSetup(gdioutput &gdi); 
  vector<ClassInfo> cInfo;

  map<int, ClassInfo> cInfoCache;

  DrawInfo drawInfo;
  void setMultiDayClass(gdioutput &gdi, bool hasMulti, TabClass::DrawMethod defaultMethod);
  void drawDialog(gdioutput &gdi, TabClass::DrawMethod method, const oClass &cls);

  void pursuitDialog(gdioutput &gdi);

  bool tableMode;
  DrawMethod lastDrawMethod;
  // Generate a table with class settings
  void showClassSettings(gdioutput &gdi);

  void visualizeField(gdioutput &gdi);

  // Read input from the table with class settings
  void readClassSettings(gdioutput &gdi);

  // Prepare for drawing by declaring starts and blocks
  void prepareForDrawing(gdioutput &gdi);


  gdioutput *gdiVisualize;

public:
  void clear();

  void closeWindow(gdioutput &gdi);

  void multiCourse(gdioutput &gdi, int nLeg);
	bool loadPage(gdioutput &gdi);
  void selectClass(gdioutput &gdi, int cid);

  int classCB(gdioutput &gdi, int type, void *data);
  int multiCB(gdioutput &gdi, int type, void *data);

	TabClass(oEvent *oe);
	~TabClass(void);
};
