/************************************************************************
    MeOS - Orienteering Software
    Copyright (C) 2009-2011 Melin Software HB
    
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Melin Software HB - software@melin.nu - www.melin.nu
    Stigbergsvgen 11, SE-75242 UPPSALA, Sweden
    
************************************************************************/

#include "stdafx.h"

#include "resource.h"

#include <commctrl.h>
#include <commdlg.h> 

#include "oEvent.h"
#include "xmlparser.h"
#include "gdioutput.h"
#include "csvparser.h"
#include "SportIdent.h"
#include <cassert>
#include "meos_util.h"
#include "Table.h"
#include "gdifonts.h"

#include "TabClub.h"

TabClub::TabClub(oEvent *poe):TabBase(poe)
{
  baseFee = 0;
  lowAge = 0;
  highAge = 0;
  filterAge = false;
  onlyNoFee = false;
  useManualFee = false;
}

TabClub::~TabClub(void)
{ 
}

void TabClub::readFeeFilter(gdioutput &gdi) {
  baseFee = oe->interpretCurrency(gdi.getText("BaseFee"));
  firstDate = gdi.getText("FirstDate");
  lastDate = gdi.getText("LastDate");
  filterAge = gdi.isChecked("FilterAge");
  useManualFee = gdi.isChecked("DefaultFees");
  if (filterAge) {
    highAge = gdi.getTextNo("HighLimit");
    lowAge = gdi.getTextNo("LowLimit");
  }

  onlyNoFee = gdi.isChecked("OnlyNoFee");

  ListBoxInfo lbi;
  gdi.getSelectedItem("ClassType", &lbi);

  if (lbi.data == -5)
    typeS = "*";
  else if (lbi.data > 0)
    typeS = "::" + itos(lbi.data);
  else
    typeS = lbi.text;

}

void TabClub::selectClub(gdioutput &gdi,  pClub pc)
{
	if (pc) {
		pc->synchronize();
    ClubId = pc->getId();
  }
	else{
    ClubId = 0;
	}
}

void manualFees(gdioutput &gdi, bool on) {
  gdi.setInputStatus("BaseFee", on);
  gdi.setInputStatus("FirstDate", on);
  gdi.setInputStatus("LastDate", on);

}

void ageFilter(gdioutput &gdi, bool on, bool use) {
  gdi.setInputStatus("HighLimit", on & use);
  gdi.setInputStatus("LowLimit", on & use);
  gdi.setInputStatus("FilterAge", use);
}

int ClubsCB(gdioutput *gdi, int type, void *data)
{
  TabClub &tc = dynamic_cast<TabClub &>(*gdi->getTabs().get(TClubTab));
  return tc.clubCB(*gdi, type, data);
}

int TabClub::clubCB(gdioutput &gdi, int type, void *data)
{
	if(type==GUI_BUTTON) {
		ButtonInfo bi=*(ButtonInfo *)data;

		if (bi.id=="Save") {
		}
		else if (bi.id=="Invoice") {
      ListBoxInfo lbi;
      gdi.getSelectedItem("Clubs", &lbi);
      pClub pc=oe->getClub(lbi.data);
      if (pc) {
        gdi.clearPage(true);
        oe->calculateResults(oEvent::RTClassResult);
        oe->sortRunners(ClassStartTime);
        int pay, paid;
        pc->generateInvoice(gdi, pc->getId(), pay, paid);
        gdi.addButton(gdi.getWidth()+20, 15, gdi.scaleLength(120), 
                      "Cancel", "terg", ClubsCB, "", true, false);
        gdi.addButton(gdi.getWidth()+20, 45,  gdi.scaleLength(120), 
                      "Print", "Skriv ut...", ClubsCB, 
                      "Skriv ut fakturan", true, false);
        gdi.refresh();
      }
		}
    else if (bi.id=="AllInvoice") {
      gdi.clearPage(false);
      oe->printInvoices(gdi, false);
      gdi.refresh();
	  gdi.print(oe, 0, false, false);
    }
    else if (bi.id=="UpdateAll") {
      oe->updateClubsFromDB();
      gdi.getTable().update();
      gdi.refresh();
    }
    else if (bi.id=="UpdateAllRunners") {
      oe->updateClubsFromDB();
      oe->updateRunnersFromDB();
      gdi.getTable().update();
      gdi.refresh();
    }
    else if (bi.id=="Update") {
      ListBoxInfo lbi;
      gdi.getSelectedItem("Clubs", &lbi);
      pClub pc=oe->getClub(lbi.data);
      if (pc) {
        pc->updateFromDB();
        pc->synchronize();
        gdi.getTable().update();
        gdi.refresh();
      }
		}    
    else if (bi.id=="Summary") {
      gdi.clearPage(false);
      oe->printInvoices(gdi, true);
      gdi.addButton(gdi.getWidth()+20, 15,  gdi.scaleLength(120), "Cancel", 
                    "terg", ClubsCB, "", true, false);
      gdi.addButton(gdi.getWidth()+20, 45,  gdi.scaleLength(120), "Print",
                    "Skriv ut...", ClubsCB, "Skriv ut fakturan", true, false);

      gdi.refresh();
		}
    else if (bi.id=="Merge") {
      ListBoxInfo lbi;
      gdi.getSelectedItem("Clubs", &lbi);
      ClubId = lbi.data;
      pClub pc=oe->getClub(lbi.data);
      if (pc) {
        gdi.clearPage(false);
        gdi.addString("", boldText, "Sl ihop klubb");
        
        char bf[256];
        sprintf_s(bf, lang.tl("help:12352").c_str(), pc->getName().c_str(), pc->getId());

        gdi.addStringUT(10, bf);

        gdi.addSelection("NewClub", 200, 300, 0, "Ny klubb:"); 
        oe->fillClubs(gdi, "NewClub");
        gdi.selectItemByData("NewClub", pc->getId());
        gdi.removeSelected("NewClub");

        gdi.pushX();
        gdi.fillRight();
        gdi.addButton("DoMerge", "Sl ihop", ClubsCB);
        gdi.addButton("Cancel", "Avbryt", ClubsCB);
        gdi.fillDown();
        gdi.popX();
        gdi.dropLine(2);
        gdi.addStringUT(boldText, lang.tl("Klubb att ta bort: ") + pc->getName());
        oe->viewClubMembers(gdi, pc->getId());

        gdi.refresh();
      }
		}    
    else if (bi.id=="DoMerge") {
      pClub pc1 = oe->getClub(ClubId);
      ListBoxInfo lbi;
      gdi.getSelectedItem("NewClub", &lbi);
      pClub pc2 = oe->getClub(lbi.data);

      if (pc1==pc2)
        throw std::exception("En klubb kan inte sls ihop med sig sjlv.");

      if (pc1 && pc2)
        oe->mergeClub(pc2->getId(), pc1->getId());
      loadPage(gdi);
    }
 		else if (bi.id=="Fees") {
      gdi.clearPage(true);

      gdi.addString("", boldLarge, "Tilldela avgifter");
      
      gdi.dropLine();
      gdi.addString("", 10, "help:assignfee");
      gdi.dropLine();
      gdi.pushX();

      gdi.addSelection("ClassType", 150, 300, 0, "Klass / klasstyp:");
      vector< pair<string, size_t> > types;
      vector< pair<string, size_t> > classes;

      oe->fillClassTypes(types);
      oe->fillClasses(classes, oEvent::extraNone, oEvent::filterNone);
      types.insert(types.end(), classes.begin(), classes.end());
      gdi.addItem("ClassType", types);
      gdi.addItem("ClassType", lang.tl("Alla typer"), -5);

      gdi.selectItemByData("ClassType", -5);

      gdi.fillRight();
      gdi.dropLine(2);
      gdi.addCheckbox("DefaultFees", "Manuella avgifter:", ClubsCB, useManualFee);

      gdi.dropLine(-1);
      
      int px = gdi.getCX();
      gdi.addInput("BaseFee", oe->formatCurrency(baseFee), 8, 0, "Avgift:");
      gdi.addInput("FirstDate", firstDate, 10, 0, "Undre datumgrns:", "-MM-DD");
      gdi.addInput("LastDate", lastDate, 10, 0, "vre datumgrns:", "-MM-DD");

      manualFees(gdi, useManualFee);

      gdi.setCX(px);
      gdi.dropLine(4);
      gdi.fillRight();
      gdi.addCheckbox("FilterAge", "ldersfilter:", ClubsCB, filterAge);

      gdi.dropLine(-1);
      gdi.addInput("LowLimit", lowAge > 0 ? itos(lowAge) : "", 5, 0, "Undre grns (r):");
      gdi.addInput("HighLimit", highAge > 0 ? itos(highAge) : "", 5, 0, "vre grns (r):");
      ageFilter(gdi, filterAge, useManualFee);

      gdi.popX();
      gdi.fillDown();
      gdi.dropLine(3);

      gdi.addCheckbox("OnlyNoFee", "Tilldela endast avgift till deltagare utan avgift", ClubsCB, onlyNoFee);
      
      gdi.pushX();
      gdi.fillRight();
      gdi.addButton("ShowFiltered", "Visa valda deltagare", ClubsCB);

      gdi.addButton("DoFees", "Tilldela avgifter", ClubsCB);
      gdi.addButton("ClearFees", "Nollstll avgifter", ClubsCB);

      gdi.addButton("Cancel", "terg", ClubsCB);
      gdi.popX();
      gdi.fillDown();
      gdi.dropLine(2);
      gdi.refresh();
		}
    else if (bi.id == "FilterAge") {
      ageFilter(gdi, gdi.isChecked(bi.id), gdi.isChecked("DefaultFees"));
    }
    else if (bi.id == "DefaultFees") {
      manualFees(gdi, gdi.isChecked(bi.id));
      ageFilter(gdi, gdi.isChecked("FilterAge"), gdi.isChecked(bi.id));
    }
  	else if (bi.id == "DoFees" || bi.id == "ClearFees" || 
             bi.id == "ShowFiltered" || bi.id == "ResetFees") {

      readFeeFilter(gdi);
      int op;

      if (bi.id == "DoFees") { 
        if (useManualFee)
          op = 0;
        else
          op = 2;
      }
      else if (bi.id == "ClearFees")
        op = 1;
      else if (bi.id == "ResetFees")
        op = 2;
      else 
        op = 3;

      gdi.restore("FeeList", false);
      gdi.setRestorePoint("FeeList");
      gdi.fillDown();

      vector<pRunner> filtered;

      oe->sortRunners(ClassStartTimeClub);
      string fdate, ldate;
      int lage = 0, hage = 0;
      
      if (useManualFee) {
        fdate = firstDate;
        ldate = lastDate;

        if (filterAge) {
          lage = lowAge;
          hage = highAge;
        }
      }
      
      oe->selectRunners(typeS, lage, hage, fdate, ldate, !onlyNoFee, filtered);
      
      gdi.dropLine(2);
      int modified = 0;
      int count = 0;
      for (size_t k = 0; k<filtered.size(); k++) {
        if (op != 1 && filtered[k]->isVacant())
          continue;
        count++;

        oDataInterface di = filtered[k]->getDI();
        int fee = 0;

        if (op == 0 || op == 1) {
          if (op == 0)
            fee = baseFee;
          
          if (di.getInt("Fee") != fee) {
            di.setInt("Fee", fee);
            modified++;
            filtered[k]->synchronize(true);
          }
        }
        else if (op == 2) {
          filtered[k]->addClassDefaultFee(true);
          if (filtered[k]->isChanged())
            modified++;
          filtered[k]->synchronize(true);
          fee = di.getInt("Fee");
        }
        else
          fee = di.getInt("Fee");

        string info = filtered[k]->getClass() + ", " + filtered[k]->getCompleteIdentification();

        gdi.addStringUT(0,  info + " (" + oe->formatCurrency(fee) + ")");
        if (count % 5 == 0)
          gdi.dropLine();
      }

      gdi.dropLine();

      if (count == 0) 
        gdi.addString("", 1, "Ingen deltagare matchar skkriteriet").setColor(colorRed);
      else if (op == 0 || op == 2)
        gdi.addString("", 1, "ndrade avgift fr X deltagare#" + itos(modified)).setColor(colorGreen);
      else if (op == 1)
        gdi.addString("", 1, "Nollstllde avgift fr X deltagare#" + itos(modified)).setColor(colorGreen);

      gdi.refresh();   
		}
		else if (bi.id=="Cancel") {
      loadPage(gdi);
		}
		else if (bi.id=="Print")	{
			gdi.print(oe);			
		}

	}
	else if(type==GUI_LISTBOX){
		ListBoxInfo bi=*(ListBoxInfo *)data;

		if(bi.id=="Clubs"){			
			pClub pc=oe->getClub(bi.data);
      if(!pc)
        throw std::exception("Internal error");

			selectClub(gdi, pc);
		}
	}

	return 0;
}


bool TabClub::loadPage(gdioutput &gdi)
{
  oe->checkDB();
	gdi.selectTab(tabId);

  if (baseFee == 0) {
    if (oe->getDCI().getInt("OrdinaryEntry") > 0)
      lastDate = oe->getDCI().getDate("OrdinaryEntry");
    baseFee = oe->getDCI().getInt("EntryFee");
    
    lowAge = 0;
    highAge = oe->getDCI().getInt("YouthAge");
  }

	gdi.clearPage(false);
	gdi.fillDown();
	gdi.addString("", boldLarge, "Klubbar");
  gdi.dropLine(0.5);
  gdi.pushX();
  gdi.fillRight();
  gdi.addSelection("Clubs", 200, 300, ClubsCB);
  oe->fillClubs(gdi, "Clubs");
  gdi.selectItemByData("Clubs", ClubId);
  gdi.addButton("Merge", "Ta bort / sl ihop...", ClubsCB);
  gdi.addButton("Invoice", "Faktura", ClubsCB);
  gdi.addButton("Update", "Uppdatera", ClubsCB, "Uppdatera klubbens uppgifter med data frn lpardatabasen/distriktsregistret");

  gdi.popX();
  gdi.dropLine(2);

  gdi.addButton("Fees", "Avgifter...", ClubsCB);
  gdi.addButton("UpdateAll", "Uppdatera alla klubbar", ClubsCB, "Uppdatera klubbarnas uppgifter med data frn lpardatabasen/distriktsregistret");
  gdi.addButton("UpdateAllRunners", "Uppdatera klubbar && lpare", ClubsCB, "Uppdatera klubbarnas och lparnas uppgifter med data frn lpardatabasen/distriktsregistret");
  
  gdi.addButton("AllInvoice", "Alla fakturor...", ClubsCB);
  gdi.addButton("Summary", "Sammanstllning", ClubsCB);
  gdi.popX();
  gdi.fillDown();
  gdi.dropLine(2);
  gdi.addString("", 10, "help:29758");
  gdi.dropLine(1);
  Table *tbl=oe->getClubsTB();
  gdi.addTable(tbl, gdi.getCX(), gdi.getCY());
  gdi.refresh();
  return true;	
}
